@@ -25,7 +25,9 @@ getIncidence <- function(cdm,
                          repeatedEvents,
                          tablePrefix,
                          returnParticipants,
-                         analysisId) {
+                         analysisId,
+                         strata,
+                         includeOverallStrata) {
   if (!is.null(outcomeWashout)) {
     if (is.na(outcomeWashout)) {
       outcomeWashout <- NULL
@@ -273,6 +275,7 @@ getIncidence <- function(cdm,
             as.Date(NA)
           ))
 
+        if(length(strata)==0 || includeOverallStrata == TRUE){
         ir[[paste0(i)]] <- workingPop %>%
           dplyr::summarise(
             n_persons = dplyr::n_distinct(.data$subject_id),
@@ -281,6 +284,24 @@ getIncidence <- function(cdm,
           ) %>%
           dplyr::mutate(incidence_start_date = .env$workingStartTime) %>%
           dplyr::mutate(incidence_end_date = .env$workingEndTime)
+        } else {
+          ir[[paste0(i)]] <- dplyr::tibble()
+        }
+
+
+        if(length(strata)>=1){
+          ir[[paste0(i)]] <- ir[[paste0(i)]] %>%
+            dplyr::mutate(strata_name = "Overall",
+                          strata_level = "Overall")
+          for(j in seq_along(strata)){
+          ir[[paste0(i)]] <-  dplyr::bind_rows(ir[[paste0(i)]],
+                             getStratifiedIncidenceResult(workingPop = workingPop,
+                                                 workingStrata = strata[[j]],
+                                                 workingStartTime = workingStartTime,
+                                                 workingEndTime= workingEndTime))
+          }
+        }
+
       }
     }
 
@@ -356,3 +377,24 @@ getIncidence <- function(cdm,
 
   return(results)
 }
+
+
+getStratifiedIncidenceResult <- function(workingPop, workingStrata,
+                                workingStartTime, workingEndTime){
+  workingPop %>%
+    dplyr::group_by(dplyr::pick(.env$workingStrata)) %>%
+    dplyr::summarise(
+      n_persons = dplyr::n_distinct(.data$subject_id),
+      person_days = sum(.data$workingDays),
+      n_events = sum(!is.na(.data$outcome_start_date))
+    ) %>%
+    dplyr::ungroup() %>%
+    dplyr::mutate(incidence_start_date = .env$workingStartTime) %>%
+    dplyr::mutate(incidence_end_date = .env$workingEndTime) %>%
+    tidyr::unite("strata_level",
+                 c(dplyr::all_of(.env$workingStrata)),
+                 remove = FALSE, sep = " and ") %>%
+    dplyr::mutate(strata_name = !!paste0(workingStrata, collapse = " and ")) %>%
+    dplyr::relocate("strata_level", .after = "strata_name") %>%
+    dplyr::select(!dplyr::any_of(workingStrata))
+}
