@@ -180,17 +180,11 @@ CDMConnector::listTables(attr(cdm, "dbcon"), schema = attr(cdm, "write_schema"))
 # drop tables created when instantiating denominator cohorts
 CDMConnector::dropTable(
   cdm = cdm,
-  name = dplyr::starts_with(paste0(
-    attr(cdm, "write_prefix"),
-    "denominator"
-  ))
+  name = dplyr::starts_with("denominator")
 )
 # drop table with study participants when returnParticipants = TRUE
 CDMConnector::dropTable(
   cdm = cdm,
-  name = paste0(
-    attr(cdm, "write_prefix"),
-    "period_prev_participants1"
-  )
+  name = "period_prev_participants1"
 )
 
